/* ------------------------------------------------------------------ */
/*      item            : ActivityLog.hxx
        generated by    : Autobuild (abuild)
        date            : Mon 27 May 2024
        category        : header file
        description     : DUECA Communication Object (DCO)
                          automatically generated by dueca-codegen
        codegen version : 111
        language        : C++
        item            : ActivityLog.dco
        made by         : Rene' van Paassen
        date            : 1301002
        description     : Log object for activity viewing
        copyright       : (c) 2016-2022 TUDelft-AE-C&S
                        : (c) 2022 René van Paassen
        license         : EUPL-1.2
*/

#pragma once
#if !defined(__DCO_NOPACK)
namespace dueca {
class AmorphStore;
class AmorphReStore;
struct DataWriterArraySize;
};
#endif

#if !defined(__DCO_STANDALONE)
namespace dueca {
struct CommObjectDataTable;
};

#include <gencodegen.h>
#if GENCODEGEN != 111
#error "Generated ActivityLog.hxx too old, please clean with 'make mrproper'"
#endif
#include <dueca/CommObjectTraits.hxx>
#endif
#include <iostream>
#include <ActivityBit.hxx>
DUECA_NS_START;
typedef uint32_t TimeTickType;
typedef ActivityBit* ActivityBitPtr;
DUECA_NS_END;


namespace dueca {
 
/** This object contains a log of the activities handled by one
    ActivityManager in one node. The log is constructed upon request
    from an ActivityView object and then sent off for feedback to the
    user.
    */
struct ActivityLog
{
  /** typedef for internal reference */
  typedef ActivityLog __ThisDCOType__;

public:
  /** The name of this class. */
  static const char* const classname;


  /** Node where this log was created */
  uint8_t node_id;

  /** Number of the ActivityManager */
  uint8_t manager_number;

  /** Value of the tick at the start of the log. All ActivityBit
      timing info is counted from this tick onwards. This to save
      space in the ActivityBit objects */
  TimeTickType base_tick;

  /** The fraction of time within a tick is indicated with a 16
      bit number in the ActivityBit objects. By multiplying this
      number with the fraction_mult, the offset in microseconds is
      recovered */
  double fraction_mult;

public:
  /** a "magic" number, hashed out of the class definition,
      that will be used to check consistency of the sent objects
      across the dueca nodes. */
  static const uint32_t magic_check_number;

  /** default constructor. */
  ActivityLog();

  /** Constructor with arguments */
  ActivityLog(
        const uint8_t& node_id,
        const uint8_t& manager_number,
        const TimeTickType& base_tick,
        const double& fraction_mult);

  /** copy constructor. */
  ActivityLog(const ActivityLog& o);

#if !defined(__DCO_NOPACK)
  /** constructor to restore an ActivityLog from amorphous storage. */
  ActivityLog(::dueca::AmorphReStore& r);
#endif

  /** destructor. */
  ~ActivityLog();

#if !defined(__DCO_STANDALONE)
  /** new operator "new", which places objects not on a
      heap, but in one of the memory arenas. This to speed up
      memory management. */
  static void* operator new(size_t size);

  /** new operator "delete", to go with the new version
      of operator new. */
  static void operator delete(void* p);

  /** placement "new", needed for stl. */
  inline static void* operator new(size_t size, ActivityLog*& o)
  { return reinterpret_cast<void*>(o); }
#endif

#if !defined(__DCO_NOPACK)
  /** packs the ActivityLog into amorphous storage. */
  void packData(::dueca::AmorphStore& s) const;

  /** packs the ActivityLog into amorphous storage.
      only differences with a previous object are packed. */
  void packDataDiff(::dueca::AmorphStore& s, const ActivityLog& ref) const;

  /** unpacks the ActivityLog from an amorphous storage. */
  void unPackData(::dueca::AmorphReStore& s);

  /** unpacks the differences for ActivityLog
      from an amorphous storage. */
  void unPackDataDiff(::dueca::AmorphReStore& s);
#endif

  /** Test for equality. */
  bool operator == (const ActivityLog& o) const;

  /** Test for inequality. */
  inline bool operator != (const ActivityLog& o) const
  { return !(*this == o); }

  /** Assignment operator. */
  ActivityLog& operator=(const ActivityLog& o);

  /** prints the ActivityLog to a stream. */
  std::ostream & print(std::ostream& s) const;

// Additional custom code
#include "ActivityLogExtra.hxx"
};

} 

#if !defined(__DCO_NOPACK)
/** pack the object into amorphous storage. */
inline void packData(::dueca::AmorphStore& s,
                     const dueca::ActivityLog& o)
{ o.packData(s); }

/** pack the differences between this object and another
    into amorphous storage. */
inline void packDataDiff(dueca::AmorphStore& s,
                         const dueca::ActivityLog& o,
                         const dueca::ActivityLog& ref)
{ o.packDataDiff(s, ref); }

/** unpack the object from amorphous storage. */
inline void unPackData(::dueca::AmorphReStore& s,
                       dueca::ActivityLog& o)
{ o.unPackData(s); }

/** unpack the differences to this object from storage. */
inline void unPackDataDiff(dueca::AmorphReStore& s,
                           dueca::ActivityLog& o)
{ o.unPackDataDiff(s); }
#endif

namespace std {
/** print to a stream. */
inline std::ostream &
operator << (std::ostream& s, const dueca::ActivityLog& o)
{ return o.print(s); }
};

// specialization of classname
template<>
const char* dueca::getclassname<dueca::ActivityLog>();
template <>
inline const char* dueca::getclassname<const dueca::ActivityLog>()
{ return dueca::getclassname<dueca::ActivityLog>(); }

#if !defined(__DCO_STANDALONE)
namespace dueca {
/** Template specialization, defines a trait that is needed if
    ActivityLog is ever used inside other dco objects. */
template <>
struct dco_nested<dueca::ActivityLog> : public dco_isnested { };
};

#endif


