/* ------------------------------------------------------------------ */
/*      item            : DUECALogConfig.hxx
        generated by    : Autobuild (abuild)
        date            : Mon 27 May 2024
        category        : header file
        description     : DUECA Communication Object (DCO)
                          automatically generated by dueca-codegen
        codegen version : 111
        language        : C++
        original item   : DUECALogConfig.dco
        made by         : Rene' van Paassen
        date            : 170521
        api             : DUECA_API
        description     : Configuration/Coordination hdf logging
        copyright       : (c) 2017-2022 TUDelft-AE-C&S
                        : (c) 2022 René van Paassen
        license         : EUPL-1.2
*/

#pragma once
#if !defined(__DCO_NOPACK)
namespace dueca {
class AmorphStore;
class AmorphReStore;
struct DataWriterArraySize;
};
#endif

#if !defined(__DCO_STANDALONE)
namespace dueca {
struct CommObjectDataTable;
};

#include <gencodegen.h>
#if GENCODEGEN != 111
#error "Generated DUECALogConfig.hxx too old, please clean with 'make mrproper'"
#endif
#include <dueca/CommObjectTraits.hxx>
#endif
#include <iostream>

#include <stringoptions.h>

namespace dueca {
 
/** Configuration message for DUECA logging.
   
    When configured to listen to config messages, the Logger will
    only start logging after such a message has been received.
   
    Possible modes of operation:
   
    - Simply configuring the attribute for the default logging; send
      one message, with filename and prefix empty, and optionally the
      attribute string filled. This uses the default filename supplied
      to the logger, and no prefix.
   
    - Multiple log files; send a message whenever a new log should be
      started, typically in HoldCurrent mode. When the filename is not
      empty, a new file is opened using the filename. When the filename
      is empty, and also the prefix is empty, a new filename is used
      with the default naming pattern.
   
    - Multiple logs in a single file (if possible with the logger); if
      the filename is empty, and the prefix is non-empty, the current
      file is used, and an additional set of logging data is added under
      the new prefix. The first message sent in this way may also specify
      the filename.
    */
struct DUECALogConfig
{
  /** typedef for internal reference */
  typedef DUECALogConfig __ThisDCOType__;

public:
  /** The name of this class. */
  static const char* const classname;


  /** File name pattern for creating a log, if empty, use previous
      or default file name.
      ;; This uses boost::posix_time to fill any patterns with time
      and/or date information. Possible patterns include %Y %m %d
      %H %M %S for year, month, day, hour, minute and second,
      respectively. */
  vstring filename;

  /** path prefix for the logging. Can be used to log multiple
      sets in a single file. If empty, no prefix is used */
  vstring prefix;

  /** attribute string that will be added to the log base point, for
      additional logging/condition information, if possible with the
      logger.
      If the prefix is empty, this is attribute is attached to the
      file root group, otherwise to the chosen prefix. */
  vstring attribute;

public:
  /** a "magic" number, hashed out of the class definition,
      that will be used to check consistency of the sent objects
      across the dueca nodes. */
  static const uint32_t magic_check_number;

  /** default constructor. */
  DUECALogConfig();

  /** Constructor with arguments */
  DUECALogConfig(
        const vstring& filename,
        const vstring& prefix,
        const vstring& attribute);

  /** copy constructor. */
  DUECALogConfig(const DUECALogConfig& o);

#if !defined(__DCO_NOPACK)
  /** constructor to restore an DUECALogConfig from amorphous storage. */
  DUECALogConfig(::dueca::AmorphReStore& r);
#endif

  /** destructor. */
  ~DUECALogConfig();

#if !defined(__DCO_STANDALONE)
  /** new operator "new", which places objects not on a
      heap, but in one of the memory arenas. This to speed up
      memory management. */
  static void* operator new(size_t size);

  /** new operator "delete", to go with the new version
      of operator new. */
  static void operator delete(void* p);

  /** placement "new", needed for stl. */
  inline static void* operator new(size_t size, DUECALogConfig*& o)
  { return reinterpret_cast<void*>(o); }
#endif

#if !defined(__DCO_NOPACK)
  /** packs the DUECALogConfig into amorphous storage. */
  void packData(::dueca::AmorphStore& s) const;

  /** packs the DUECALogConfig into amorphous storage.
      only differences with a previous object are packed. */
  void packDataDiff(::dueca::AmorphStore& s, const DUECALogConfig& ref) const;

  /** unpacks the DUECALogConfig from an amorphous storage. */
  void unPackData(::dueca::AmorphReStore& s);

  /** unpacks the differences for DUECALogConfig
      from an amorphous storage. */
  void unPackDataDiff(::dueca::AmorphReStore& s);
#endif

  /** Test for equality. */
  bool operator == (const DUECALogConfig& o) const;

  /** Test for inequality. */
  inline bool operator != (const DUECALogConfig& o) const
  { return !(*this == o); }

  /** Assignment operator. */
  DUECALogConfig& operator=(const DUECALogConfig& o);

  /** prints the DUECALogConfig to a stream. */
  std::ostream & print(std::ostream& s) const;
};

} 

#if !defined(__DCO_NOPACK)
/** pack the object into amorphous storage. */
inline void packData(::dueca::AmorphStore& s,
                     const dueca::DUECALogConfig& o)
{ o.packData(s); }

/** pack the differences between this object and another
    into amorphous storage. */
inline void packDataDiff(dueca::AmorphStore& s,
                         const dueca::DUECALogConfig& o,
                         const dueca::DUECALogConfig& ref)
{ o.packDataDiff(s, ref); }

/** unpack the object from amorphous storage. */
inline void unPackData(::dueca::AmorphReStore& s,
                       dueca::DUECALogConfig& o)
{ o.unPackData(s); }

/** unpack the differences to this object from storage. */
inline void unPackDataDiff(dueca::AmorphReStore& s,
                           dueca::DUECALogConfig& o)
{ o.unPackDataDiff(s); }
#endif

namespace std {
/** print to a stream. */
inline std::ostream &
operator << (std::ostream& s, const dueca::DUECALogConfig& o)
{ return o.print(s); }
};

// specialization of classname
template<>
const char* dueca::getclassname<dueca::DUECALogConfig>();
template <>
inline const char* dueca::getclassname<const dueca::DUECALogConfig>()
{ return dueca::getclassname<dueca::DUECALogConfig>(); }

#if !defined(__DCO_STANDALONE)
namespace dueca {
/** Template specialization, defines a trait that is needed if
    DUECALogConfig is ever used inside other dco objects. */
template <>
struct dco_nested<dueca::DUECALogConfig> : public dco_isnested { };
};

#endif


