/* ------------------------------------------------------------------ */
/*      item            : LogPoint.hxx
        generated by    : Autobuild (abuild)
        date            : Mon 27 May 2024
        category        : header file
        description     : DUECA Communication Object (DCO)
                          automatically generated by dueca-codegen
        codegen version : 111
        language        : C++
        item            : LogPoint.dco
        made by         : Rene' van Paassen
        date            : 1301002
        description     : Define a logging location
        copyright       : (c) 2016-2022 TUDelft-AE-C&S
                        : (c) 2022 René van Paassen
        license         : EUPL-1.2
*/

#pragma once
#if !defined(__DCO_NOPACK)
namespace dueca {
class AmorphStore;
class AmorphReStore;
struct DataWriterArraySize;
};
#endif

#if !defined(__DCO_STANDALONE)
namespace dueca {
struct CommObjectDataTable;
};

#include <gencodegen.h>
#if GENCODEGEN != 111
#error "Generated LogPoint.hxx too old, please clean with 'make mrproper'"
#endif
#include <dueca/CommObjectTraits.hxx>
#endif
#include <iostream>

#include <LogLevel.hxx>
#include <stringoptions.h>

namespace dueca {
 
/** This event describes a log point somewhere in a DUECA executables.
    These log points may be coming from the DUECA system, or from user code.
    The old D_MOD etc macros still work for these log points. Trimming them
    may be useful, since file, line number and reporter/associate
    information is added automatically.  the node information is implicit,
    keep LogPoints from different nodes apart.
    */
struct LogPoint
{
  /** typedef for internal reference */
  typedef LogPoint __ThisDCOType__;

public:
  /** The name of this class. */
  static const char* const classname;


  /** logpoint id. */
  uint32_t logpoint;

  /** line in the file */
  uint32_t line;

  /** log level */
  LogLevel level;

  /** log category */
  uint32_t category;

  /** file name */
  string64 fname;

public:
  /** a "magic" number, hashed out of the class definition,
      that will be used to check consistency of the sent objects
      across the dueca nodes. */
  static const uint32_t magic_check_number;

  /** default constructor. */
  LogPoint();

  /** Constructor with arguments */
  LogPoint(
        const uint32_t& logpoint,
        const uint32_t& line,
        const LogLevel& level,
        const uint32_t& category,
        const string64& fname);

  /** copy constructor. */
  LogPoint(const LogPoint& o);

#if !defined(__DCO_NOPACK)
  /** constructor to restore an LogPoint from amorphous storage. */
  LogPoint(::dueca::AmorphReStore& r);
#endif

  /** destructor. */
  ~LogPoint();

#if !defined(__DCO_STANDALONE)
  /** new operator "new", which places objects not on a
      heap, but in one of the memory arenas. This to speed up
      memory management. */
  static void* operator new(size_t size);

  /** new operator "delete", to go with the new version
      of operator new. */
  static void operator delete(void* p);

  /** placement "new", needed for stl. */
  inline static void* operator new(size_t size, LogPoint*& o)
  { return reinterpret_cast<void*>(o); }
#endif

#if !defined(__DCO_NOPACK)
  /** packs the LogPoint into amorphous storage. */
  void packData(::dueca::AmorphStore& s) const;

  /** packs the LogPoint into amorphous storage.
      only differences with a previous object are packed. */
  void packDataDiff(::dueca::AmorphStore& s, const LogPoint& ref) const;

  /** unpacks the LogPoint from an amorphous storage. */
  void unPackData(::dueca::AmorphReStore& s);

  /** unpacks the differences for LogPoint
      from an amorphous storage. */
  void unPackDataDiff(::dueca::AmorphReStore& s);
#endif

  /** Test for equality. */
  bool operator == (const LogPoint& o) const;

  /** Test for inequality. */
  inline bool operator != (const LogPoint& o) const
  { return !(*this == o); }

  /** Assignment operator. */
  LogPoint& operator=(const LogPoint& o);

  /** prints the LogPoint to a stream. */
  std::ostream & print(std::ostream& s) const;
};

} 

#if !defined(__DCO_NOPACK)
/** pack the object into amorphous storage. */
inline void packData(::dueca::AmorphStore& s,
                     const dueca::LogPoint& o)
{ o.packData(s); }

/** pack the differences between this object and another
    into amorphous storage. */
inline void packDataDiff(dueca::AmorphStore& s,
                         const dueca::LogPoint& o,
                         const dueca::LogPoint& ref)
{ o.packDataDiff(s, ref); }

/** unpack the object from amorphous storage. */
inline void unPackData(::dueca::AmorphReStore& s,
                       dueca::LogPoint& o)
{ o.unPackData(s); }

/** unpack the differences to this object from storage. */
inline void unPackDataDiff(dueca::AmorphReStore& s,
                           dueca::LogPoint& o)
{ o.unPackDataDiff(s); }
#endif

namespace std {
/** print to a stream. */
inline std::ostream &
operator << (std::ostream& s, const dueca::LogPoint& o)
{ return o.print(s); }
};

// specialization of classname
template<>
const char* dueca::getclassname<dueca::LogPoint>();
template <>
inline const char* dueca::getclassname<const dueca::LogPoint>()
{ return dueca::getclassname<dueca::LogPoint>(); }

#if !defined(__DCO_STANDALONE)
namespace dueca {
/** Template specialization, defines a trait that is needed if
    LogPoint is ever used inside other dco objects. */
template <>
struct dco_nested<dueca::LogPoint> : public dco_isnested { };
};

#endif


