/* ------------------------------------------------------------------ */
/*      item            : NodeControlMessage.hxx
        generated by    : Autobuild (abuild)
        date            : Mon 27 May 2024
        category        : header file
        description     : DUECA Communication Object (DCO)
                          automatically generated by dueca-codegen
        codegen version : 111
        language        : C++
        item            : NodeControlMessage.dco
        made by         : Rene' van Paassen
        date            : 1301002
        description     : distributed state machine
        copyright       : (c) 2016-2022 TUDelft-AE-C&S
                        : (c) 2022 René van Paassen
        license         : EUPL-1.2
*/

#pragma once
#if !defined(__DCO_NOPACK)
namespace dueca {
class AmorphStore;
class AmorphReStore;
struct DataWriterArraySize;
};
#endif

#if !defined(__DCO_STANDALONE)
namespace dueca {
struct CommObjectDataTable;
};

#include <gencodegen.h>
#if GENCODEGEN != 111
#error "Generated NodeControlMessage.hxx too old, please clean with 'make mrproper'"
#endif
#include <dueca/CommObjectTraits.hxx>
#endif
#include <iostream>



namespace dueca {
 
/** Control message for commanding/feedback of node state
    */
struct NodeControlMessage
{
  /** typedef for internal reference */
  typedef NodeControlMessage __ThisDCOType__;

public:
  /** The name of this class. */
  static const char* const classname;


/** Node state, serves as a report type, for communication between

    node managers
 */
  enum NodeState {
    /** node is unconnected */
    NodeLoose,
    /** node is trying to connect */
    NodeJoining,
    /** node has joined the dueca process */
    NodeJoined,
    /** node timing has been synchronized */
    NodeSynced,
    /** node is breaking off */
    NodeBreak,
    /** node is breaking off */
    NodeBreaking,
    /** node query command */
    NodeQuery,
    /** emergency occurred, node to safe state and stopped communication */
    Emergency
  };

  /** requested node state */
  NodeState state;

public:
  /** a "magic" number, hashed out of the class definition,
      that will be used to check consistency of the sent objects
      across the dueca nodes. */
  static const uint32_t magic_check_number;

  /** default constructor. */
  NodeControlMessage();

  /** Constructor with arguments */
  NodeControlMessage(
        const NodeState& state);

  /** copy constructor. */
  NodeControlMessage(const NodeControlMessage& o);

#if !defined(__DCO_NOPACK)
  /** constructor to restore an NodeControlMessage from amorphous storage. */
  NodeControlMessage(::dueca::AmorphReStore& r);
#endif

  /** destructor. */
  ~NodeControlMessage();

#if !defined(__DCO_STANDALONE)
  /** new operator "new", which places objects not on a
      heap, but in one of the memory arenas. This to speed up
      memory management. */
  static void* operator new(size_t size);

  /** new operator "delete", to go with the new version
      of operator new. */
  static void operator delete(void* p);

  /** placement "new", needed for stl. */
  inline static void* operator new(size_t size, NodeControlMessage*& o)
  { return reinterpret_cast<void*>(o); }
#endif

#if !defined(__DCO_NOPACK)
  /** packs the NodeControlMessage into amorphous storage. */
  void packData(::dueca::AmorphStore& s) const;

  /** packs the NodeControlMessage into amorphous storage.
      only differences with a previous object are packed. */
  void packDataDiff(::dueca::AmorphStore& s, const NodeControlMessage& ref) const;

  /** unpacks the NodeControlMessage from an amorphous storage. */
  void unPackData(::dueca::AmorphReStore& s);

  /** unpacks the differences for NodeControlMessage
      from an amorphous storage. */
  void unPackDataDiff(::dueca::AmorphReStore& s);
#endif

  /** Test for equality. */
  bool operator == (const NodeControlMessage& o) const;

  /** Test for inequality. */
  inline bool operator != (const NodeControlMessage& o) const
  { return !(*this == o); }

  /** Assignment operator. */
  NodeControlMessage& operator=(const NodeControlMessage& o);

  /** prints the NodeControlMessage to a stream. */
  std::ostream & print(std::ostream& s) const;
};

} 

#if !defined(__DCO_NOPACK)
/** pack the object into amorphous storage. */
inline void packData(::dueca::AmorphStore& s,
                     const dueca::NodeControlMessage& o)
{ o.packData(s); }

/** pack the differences between this object and another
    into amorphous storage. */
inline void packDataDiff(dueca::AmorphStore& s,
                         const dueca::NodeControlMessage& o,
                         const dueca::NodeControlMessage& ref)
{ o.packDataDiff(s, ref); }

/** unpack the object from amorphous storage. */
inline void unPackData(::dueca::AmorphReStore& s,
                       dueca::NodeControlMessage& o)
{ o.unPackData(s); }

/** unpack the differences to this object from storage. */
inline void unPackDataDiff(dueca::AmorphReStore& s,
                           dueca::NodeControlMessage& o)
{ o.unPackDataDiff(s); }
#endif

namespace dueca {
  const char* const getString(const dueca::NodeControlMessage::NodeState &o);
  void readFromString(dueca::NodeControlMessage::NodeState &o, const std::string& s);
  void getFirst(dueca::NodeControlMessage::NodeState &o);
  bool getNext(dueca::NodeControlMessage::NodeState &o);

  /** This function retrieves the classname of an NodeControlMessage::NodeState */
  template <>
  const char* getclassname<dueca::NodeControlMessage::NodeState>();
}; // end namespace dueca

#if !defined(__DCO_NOPACK)
void packData(::dueca::AmorphStore& s,
              const dueca::NodeControlMessage::NodeState &o);
void unPackData(::dueca::AmorphReStore& s,
                dueca::NodeControlMessage::NodeState &o);
#endif

PRINT_NS_START;
/** Print an object of type NodeControlMessage::NodeState */
inline std::ostream&
operator << (std::ostream& s, const dueca::NodeControlMessage::NodeState& o)
{ return s << dueca::getString(o); }
/** Read an object of type NodeControlMessage::NodeState */
inline std::istream&
/** Read an enum of type NodeControlMessage::NodeState */
operator >> (std::istream& s, dueca::NodeControlMessage::NodeState& o)
{ std::string tmp; s >> tmp; dueca::readFromString(o, tmp); return s; }
PRINT_NS_END;

namespace std {
/** print to a stream. */
inline std::ostream &
operator << (std::ostream& s, const dueca::NodeControlMessage& o)
{ return o.print(s); }
};

// specialization of classname
template<>
const char* dueca::getclassname<dueca::NodeControlMessage>();
template <>
inline const char* dueca::getclassname<const dueca::NodeControlMessage>()
{ return dueca::getclassname<dueca::NodeControlMessage>(); }

#if !defined(__DCO_STANDALONE)
namespace dueca {
/** Template specialization, defines a trait that is needed if
    NodeControlMessage is ever used inside other dco objects. */
template <>
struct dco_nested<dueca::NodeControlMessage> : public dco_isnested { };
template <>
struct dco_nested<dueca::NodeControlMessage::NodeState> : public dco_isenum { };
};

#endif


