/* ------------------------------------------------------------------ */
/*      item            : SimStateRequest.hxx
        generated by    : Autobuild (abuild)
        date            : Mon 27 May 2024
        category        : header file
        description     : DUECA Communication Object (DCO)
                          automatically generated by dueca-codegen
        codegen version : 111
        language        : C++
        item            : SimStateRequest.dco
        made by         : Rene' van Paassen
        date            : 1301002
        description     : Generated from a hard-coded version
        copyright       : (c) 2016-2022 TUDelft-AE-C&S
                        : (c) 2022 René van Paassen
        license         : EUPL-1.2
*/

#pragma once
#if !defined(__DCO_NOPACK)
namespace dueca {
class AmorphStore;
class AmorphReStore;
struct DataWriterArraySize;
};
#endif

#if !defined(__DCO_STANDALONE)
namespace dueca {
struct CommObjectDataTable;
};

#include <gencodegen.h>
#if GENCODEGEN != 111
#error "Generated SimStateRequest.hxx too old, please clean with 'make mrproper'"
#endif
#include <dueca/CommObjectTraits.hxx>
#endif
#include <iostream>

#include <SimulationState.hxx>

namespace dueca {
 
/** For requesting specific state changes of the DUSIME control process.
   
    The SimStateRequest channel is a channel with event data, and with zero
    or more entries. A receiving token is defined by the DUSIME code.
   
    You can use this channel to request changes in the DUSIME
    state machine. In your header file, declare a writing token:
    \code
    ChannelWriteToken                w_state_request;
    \endcode
    In your constructor, initialize the token with
    \code
    w_state_request(getId(), NameSet("dusime", "SimStateRequest", ""),
                    "SimStateRequest", "", Channel::Events,
                    Channel::ZeroOrMoreEntries),
    \endcode
   
    At the appropriate moment, you can request a DUSIME state change
    with the desired final state (HoldCurrent, Advance, Replay or Inactive)
    as data in the request. Note that state changes, due to the nature of
    DUECA's distributed processing, cannot be instantaneous. The effect of
    sending the state change would be that of pressing the button at that
    moment.
   
    \code
    {
      // switching back to HoldCurrent
      DataWriter<SimStateRequest> wr(w_state_request, ts);
      wr.data().request = SimulationState::HoldCurrent;
    }
    \endcode
    */
struct SimStateRequest
{
  /** typedef for internal reference */
  typedef SimStateRequest __ThisDCOType__;

public:
  /** The name of this class. */
  static const char* const classname;


  /** requested simulation state */
  SimulationState request;

public:
  /** a "magic" number, hashed out of the class definition,
      that will be used to check consistency of the sent objects
      across the dueca nodes. */
  static const uint32_t magic_check_number;

  /** default constructor. */
  SimStateRequest();

  /** Constructor with arguments */
  SimStateRequest(
        const SimulationState& request);

  /** copy constructor. */
  SimStateRequest(const SimStateRequest& o);

#if !defined(__DCO_NOPACK)
  /** constructor to restore an SimStateRequest from amorphous storage. */
  SimStateRequest(::dueca::AmorphReStore& r);
#endif

  /** destructor. */
  ~SimStateRequest();

#if !defined(__DCO_STANDALONE)
  /** new operator "new", which places objects not on a
      heap, but in one of the memory arenas. This to speed up
      memory management. */
  static void* operator new(size_t size);

  /** new operator "delete", to go with the new version
      of operator new. */
  static void operator delete(void* p);

  /** placement "new", needed for stl. */
  inline static void* operator new(size_t size, SimStateRequest*& o)
  { return reinterpret_cast<void*>(o); }
#endif

#if !defined(__DCO_NOPACK)
  /** packs the SimStateRequest into amorphous storage. */
  void packData(::dueca::AmorphStore& s) const;

  /** packs the SimStateRequest into amorphous storage.
      only differences with a previous object are packed. */
  void packDataDiff(::dueca::AmorphStore& s, const SimStateRequest& ref) const;

  /** unpacks the SimStateRequest from an amorphous storage. */
  void unPackData(::dueca::AmorphReStore& s);

  /** unpacks the differences for SimStateRequest
      from an amorphous storage. */
  void unPackDataDiff(::dueca::AmorphReStore& s);
#endif

  /** Test for equality. */
  bool operator == (const SimStateRequest& o) const;

  /** Test for inequality. */
  inline bool operator != (const SimStateRequest& o) const
  { return !(*this == o); }

  /** Assignment operator. */
  SimStateRequest& operator=(const SimStateRequest& o);

  /** prints the SimStateRequest to a stream. */
  std::ostream & print(std::ostream& s) const;
};

} 

#if !defined(__DCO_NOPACK)
/** pack the object into amorphous storage. */
inline void packData(::dueca::AmorphStore& s,
                     const dueca::SimStateRequest& o)
{ o.packData(s); }

/** pack the differences between this object and another
    into amorphous storage. */
inline void packDataDiff(dueca::AmorphStore& s,
                         const dueca::SimStateRequest& o,
                         const dueca::SimStateRequest& ref)
{ o.packDataDiff(s, ref); }

/** unpack the object from amorphous storage. */
inline void unPackData(::dueca::AmorphReStore& s,
                       dueca::SimStateRequest& o)
{ o.unPackData(s); }

/** unpack the differences to this object from storage. */
inline void unPackDataDiff(dueca::AmorphReStore& s,
                           dueca::SimStateRequest& o)
{ o.unPackDataDiff(s); }
#endif

namespace std {
/** print to a stream. */
inline std::ostream &
operator << (std::ostream& s, const dueca::SimStateRequest& o)
{ return o.print(s); }
};

// specialization of classname
template<>
const char* dueca::getclassname<dueca::SimStateRequest>();
template <>
inline const char* dueca::getclassname<const dueca::SimStateRequest>()
{ return dueca::getclassname<dueca::SimStateRequest>(); }

#if !defined(__DCO_STANDALONE)
namespace dueca {
/** Template specialization, defines a trait that is needed if
    SimStateRequest is ever used inside other dco objects. */
template <>
struct dco_nested<dueca::SimStateRequest> : public dco_isnested { };
};

#endif


