/* ------------------------------------------------------------------ */
/*      item            : UChannelCommRequest.hxx
        generated by    : Autobuild (abuild)
        date            : Mon 27 May 2024
        category        : header file
        description     : DUECA Communication Object (DCO)
                          automatically generated by dueca-codegen
        codegen version : 111
        language        : C++
        original item   : UChannelCommRequest.dco
        made by         : Rene' van Paassen
        date            : 131230
        description     : Communication/coordination for UnifiedChannel
        copyright       : (c) 2016-2022 TUDelft-AE-C&S
                        : (c) 2022 René van Paassen
        license         : EUPL-1.2
*/

#pragma once
#if !defined(__DCO_NOPACK)
namespace dueca {
class AmorphStore;
class AmorphReStore;
struct DataWriterArraySize;
};
#endif

#if !defined(__DCO_STANDALONE)
namespace dueca {
struct CommObjectDataTable;
};

#include <gencodegen.h>
#if GENCODEGEN != 111
#error "Generated UChannelCommRequest.hxx too old, please clean with 'make mrproper'"
#endif
#include <dueca/CommObjectTraits.hxx>
#endif
#include <iostream>

#include <GlobalId.hxx>
#include <string>

namespace dueca {
 
/** Class that defines the interpretation by MultiStreamChannel objects
    for the following data
    */
struct UChannelCommRequest
{
  /** typedef for internal reference */
  typedef UChannelCommRequest __ThisDCOType__;

public:
  /** The name of this class. */
  static const char* const classname;


/** Enumerated value that signifies the message type in the packed data.

    TODO: maybe merge the conf and req variants, anything going out from

          master seen as command, anything coming in as request
 */
  enum UChannelMessageType {
    /** Rest of the message contains differential data wrt the
        previously communicated data. */
    DiffData,
    /** Rest of the message contains a full set of data. */
    FullData,
    /** Control message, indicates a request for a full data set
        for a specific entry */
    FullDataReq,
    /** Message to remove the saveup */
    RemoveSaveupCmd,
    /** Create a jump in contiguous time */
    TimeJump,
    /** Command to clean entry */
    CleanEntryCmd,
    /** Command to invalidate an entry */
    InvalidateEntryCmd,
    /** Control message, indicating that the new entry has an entry
        number and can be used. Indicates also to other ends that
        room has to be reserved for data entering here. */
    NewEntryConf,
    /** Response by the master to a new end joining, telling the
        number of entries currently in use */
    NewEndWelcome,
    /** Command to delete entry */
    DeleteEntryCmd,
    /** Control message, indicating a specific node has a new entry
        lined up. Will be replied to with a NewEntryConf by the
        master, giving the entry id. */
    NewEntryReq,
    /** Request to delete and clean an entry */
    DeleteEntryReq,
    /** Confirmation that entry is clean */
    CleanEntryConf,
    /** Notification that a new reading client is present. The
        master does not confirm this, but uses the notifications too
        keep inventory of the number of clients and issue a
        RemoveSaveUpCmd when all reservations have been used */
    NewClientNotif,
    /** Notification of a reading client leaving */
    LeaveClientNotif,
    /** Control message, indicates that an entry is being used in a
        certain node. Entry use is being counted. */
    NewEndJoins,
    /** Dummy message, for just in case. Do not react to. */
    DummyMessage
  };

  /** Enumerated value that gives the type */
  UChannelMessageType type;

  /** extra flags? */
  uint8_t extra;

  /** Unsigned integer value that sometimes is needed for
      further interpretation */
  uint16_t data0;

  /** If needed, an identification */
  uint32_t data1;

  /** Origin/writer of the data, only relevant for
      NewEntryReq and EntryConf */
  GlobalId origin;

  /** name of the data class type, only relevant for
      NewEntryReq and EntryConf */
  std::string dataclassname;

  /** label for this entry, only relevant for
      NewEntryReq and EntryConf */
  std::string entrylabel;

public:
  /** a "magic" number, hashed out of the class definition,
      that will be used to check consistency of the sent objects
      across the dueca nodes. */
  static const uint32_t magic_check_number;

  /** default constructor. */
  UChannelCommRequest(
        const UChannelMessageType& type = DummyMessage,
        const uint8_t& extra = 0U,
        const uint16_t& data0 = 0U,
        const uint32_t& data1 = 0U,
        const std::string& dataclassname = std::string(),
        const std::string& entrylabel = std::string());

  /** Constructor with arguments */
  UChannelCommRequest(
        const UChannelMessageType& type,
        const uint8_t& extra,
        const uint16_t& data0,
        const uint32_t& data1,
        const GlobalId& origin,
        const std::string& dataclassname,
        const std::string& entrylabel);

  /** copy constructor. */
  UChannelCommRequest(const UChannelCommRequest& o);

#if !defined(__DCO_NOPACK)
  /** constructor to restore an UChannelCommRequest from amorphous storage. */
  UChannelCommRequest(::dueca::AmorphReStore& r);
#endif

  /** destructor. */
  ~UChannelCommRequest();

#if !defined(__DCO_STANDALONE)
  /** new operator "new", which places objects not on a
      heap, but in one of the memory arenas. This to speed up
      memory management. */
  static void* operator new(size_t size);

  /** new operator "delete", to go with the new version
      of operator new. */
  static void operator delete(void* p);

  /** placement "new", needed for stl. */
  inline static void* operator new(size_t size, UChannelCommRequest*& o)
  { return reinterpret_cast<void*>(o); }
#endif

#if !defined(__DCO_NOPACK)
  /** packs the UChannelCommRequest into amorphous storage. */
  void packData(::dueca::AmorphStore& s) const;

  /** packs the UChannelCommRequest into amorphous storage.
      only differences with a previous object are packed. */
  void packDataDiff(::dueca::AmorphStore& s, const UChannelCommRequest& ref) const;

  /** unpacks the UChannelCommRequest from an amorphous storage. */
  void unPackData(::dueca::AmorphReStore& s);

  /** unpacks the differences for UChannelCommRequest
      from an amorphous storage. */
  void unPackDataDiff(::dueca::AmorphReStore& s);
#endif

  /** Test for equality. */
  bool operator == (const UChannelCommRequest& o) const;

  /** Test for inequality. */
  inline bool operator != (const UChannelCommRequest& o) const
  { return !(*this == o); }

  /** Assignment operator. */
  UChannelCommRequest& operator=(const UChannelCommRequest& o);

  /** prints the UChannelCommRequest to a stream. */
  std::ostream & print(std::ostream& s) const;

// Additional custom code
#include "UChannelCommRequestExtra.hxx"
};

} 

#if !defined(__DCO_NOPACK)
/** pack the object into amorphous storage. */
inline void packData(::dueca::AmorphStore& s,
                     const dueca::UChannelCommRequest& o)
{ o.packData(s); }

/** pack the differences between this object and another
    into amorphous storage. */
inline void packDataDiff(dueca::AmorphStore& s,
                         const dueca::UChannelCommRequest& o,
                         const dueca::UChannelCommRequest& ref)
{ o.packDataDiff(s, ref); }

/** unpack the object from amorphous storage. */
inline void unPackData(::dueca::AmorphReStore& s,
                       dueca::UChannelCommRequest& o)
{ o.unPackData(s); }

/** unpack the differences to this object from storage. */
inline void unPackDataDiff(dueca::AmorphReStore& s,
                           dueca::UChannelCommRequest& o)
{ o.unPackDataDiff(s); }
#endif

namespace dueca {
  const char* const getString(const dueca::UChannelCommRequest::UChannelMessageType &o);
  void readFromString(dueca::UChannelCommRequest::UChannelMessageType &o, const std::string& s);
  void getFirst(dueca::UChannelCommRequest::UChannelMessageType &o);
  bool getNext(dueca::UChannelCommRequest::UChannelMessageType &o);

  /** This function retrieves the classname of an UChannelCommRequest::UChannelMessageType */
  template <>
  const char* getclassname<dueca::UChannelCommRequest::UChannelMessageType>();
}; // end namespace dueca

#if !defined(__DCO_NOPACK)
void packData(::dueca::AmorphStore& s,
              const dueca::UChannelCommRequest::UChannelMessageType &o);
void unPackData(::dueca::AmorphReStore& s,
                dueca::UChannelCommRequest::UChannelMessageType &o);
#endif

PRINT_NS_START;
/** Print an object of type UChannelCommRequest::UChannelMessageType */
inline std::ostream&
operator << (std::ostream& s, const dueca::UChannelCommRequest::UChannelMessageType& o)
{ return s << dueca::getString(o); }
/** Read an object of type UChannelCommRequest::UChannelMessageType */
inline std::istream&
/** Read an enum of type UChannelCommRequest::UChannelMessageType */
operator >> (std::istream& s, dueca::UChannelCommRequest::UChannelMessageType& o)
{ std::string tmp; s >> tmp; dueca::readFromString(o, tmp); return s; }
PRINT_NS_END;

namespace std {
/** print to a stream. */
inline std::ostream &
operator << (std::ostream& s, const dueca::UChannelCommRequest& o)
{ return o.print(s); }
};

// specialization of classname
template<>
const char* dueca::getclassname<dueca::UChannelCommRequest>();
template <>
inline const char* dueca::getclassname<const dueca::UChannelCommRequest>()
{ return dueca::getclassname<dueca::UChannelCommRequest>(); }

#if !defined(__DCO_STANDALONE)
namespace dueca {
/** Template specialization, defines a trait that is needed if
    UChannelCommRequest is ever used inside other dco objects. */
template <>
struct dco_nested<dueca::UChannelCommRequest> : public dco_isnested { };
template <>
struct dco_nested<dueca::UChannelCommRequest::UChannelMessageType> : public dco_isenum { };
};

#endif


