/* ------------------------------------------------------------------ */
/*      item            : ReplayReport.hxx
        generated by    : Autobuild (abuild)
        date            : Mon 27 May 2024
        category        : header file
        description     : DUECA Communication Object (DCO)
                          automatically generated by dueca-codegen
        codegen version : 111
        language        : C++
        item            : ReplayReport.dco
        made by         : Rene' van Paassen
        date            : 220110
        description     : Report to control data replay
        copyright       : (c) 2022-2022 TUDelft-AE-C&S
                        : (c) 2022 René van Paassen
        license         : EUPL-1.2
*/

#pragma once
#if !defined(__DCO_NOPACK)
namespace dueca {
class AmorphStore;
class AmorphReStore;
struct DataWriterArraySize;
};
#endif

#if !defined(__DCO_STANDALONE)
namespace dueca {
struct CommObjectDataTable;
};

#include <gencodegen.h>
#if GENCODEGEN != 111
#error "Generated ReplayReport.hxx too old, please clean with 'make mrproper'"
#endif
#include <dueca/CommObjectTraits.hxx>
#endif
#include <iostream>

#include <inttypes.h>
#include <string>
#include <dueca/DataTimeSpec.hxx>

namespace dueca {
 
/** This command controls the data replay preparation
   
    These are sent by the ReplayFiler objects (one per node where the
    associated entity stores replay data). After opening a channel, the
    existing recording information is sent. Then an InitFiler message
    indicates all initial information is complete.
    When a new recording is made, more TagInformation messages are sent
    */
struct ReplayReport
{
  /** typedef for internal reference */
  typedef ReplayReport __ThisDCOType__;

public:
  /** The name of this class. */
  static const char* const classname;


/** Enumerated type for an automatically generated object class */
  enum class Status {
    /** Information on logged tags */
    TagInformation,
    /** Performed an init of the filers */
    InitFiler,
    /** Error occurred */
    Error
  };

  /** The command (currently only one option) */
  Status status;

  /** Numeric for the tag info */
  uint32_t number;

  /** Run generation to select for the data */
  std::string label;

  /** Additional data */
  std::string time;

  /** If applicable, start of the recording */
  TimeTickType tick0;

  /** If applicable, end of the recording */
  TimeTickType tick1;

  /** And inco information if available */
  std::string inco_name;

public:
  /** a "magic" number, hashed out of the class definition,
      that will be used to check consistency of the sent objects
      across the dueca nodes. */
  static const uint32_t magic_check_number;

  /** default constructor. */
  ReplayReport();

  /** Constructor with arguments */
  ReplayReport(
        const Status& status,
        const uint32_t& number,
        const std::string& label,
        const std::string& time,
        const TimeTickType& tick0,
        const TimeTickType& tick1,
        const std::string& inco_name);

  /** copy constructor. */
  ReplayReport(const ReplayReport& o);

#if !defined(__DCO_NOPACK)
  /** constructor to restore an ReplayReport from amorphous storage. */
  ReplayReport(::dueca::AmorphReStore& r);
#endif

  /** destructor. */
  ~ReplayReport();

#if !defined(__DCO_STANDALONE)
  /** new operator "new", which places objects not on a
      heap, but in one of the memory arenas. This to speed up
      memory management. */
  static void* operator new(size_t size);

  /** new operator "delete", to go with the new version
      of operator new. */
  static void operator delete(void* p);

  /** placement "new", needed for stl. */
  inline static void* operator new(size_t size, ReplayReport*& o)
  { return reinterpret_cast<void*>(o); }
#endif

#if !defined(__DCO_NOPACK)
  /** packs the ReplayReport into amorphous storage. */
  void packData(::dueca::AmorphStore& s) const;

  /** packs the ReplayReport into amorphous storage.
      only differences with a previous object are packed. */
  void packDataDiff(::dueca::AmorphStore& s, const ReplayReport& ref) const;

  /** unpacks the ReplayReport from an amorphous storage. */
  void unPackData(::dueca::AmorphReStore& s);

  /** unpacks the differences for ReplayReport
      from an amorphous storage. */
  void unPackDataDiff(::dueca::AmorphReStore& s);
#endif

  /** Test for equality. */
  bool operator == (const ReplayReport& o) const;

  /** Test for inequality. */
  inline bool operator != (const ReplayReport& o) const
  { return !(*this == o); }

  /** Assignment operator. */
  ReplayReport& operator=(const ReplayReport& o);

  /** prints the ReplayReport to a stream. */
  std::ostream & print(std::ostream& s) const;
};

} 

#if !defined(__DCO_NOPACK)
/** pack the object into amorphous storage. */
inline void packData(::dueca::AmorphStore& s,
                     const dueca::ReplayReport& o)
{ o.packData(s); }

/** pack the differences between this object and another
    into amorphous storage. */
inline void packDataDiff(dueca::AmorphStore& s,
                         const dueca::ReplayReport& o,
                         const dueca::ReplayReport& ref)
{ o.packDataDiff(s, ref); }

/** unpack the object from amorphous storage. */
inline void unPackData(::dueca::AmorphReStore& s,
                       dueca::ReplayReport& o)
{ o.unPackData(s); }

/** unpack the differences to this object from storage. */
inline void unPackDataDiff(dueca::AmorphReStore& s,
                           dueca::ReplayReport& o)
{ o.unPackDataDiff(s); }
#endif

namespace dueca {
  const char* const getString(const dueca::ReplayReport::Status &o);
  void readFromString(dueca::ReplayReport::Status &o, const std::string& s);
  void getFirst(dueca::ReplayReport::Status &o);
  bool getNext(dueca::ReplayReport::Status &o);

  /** This function retrieves the classname of an ReplayReport::Status */
  template <>
  const char* getclassname<dueca::ReplayReport::Status>();
}; // end namespace dueca

#if !defined(__DCO_NOPACK)
void packData(::dueca::AmorphStore& s,
              const dueca::ReplayReport::Status &o);
void unPackData(::dueca::AmorphReStore& s,
                dueca::ReplayReport::Status &o);
#endif

PRINT_NS_START;
/** Print an object of type ReplayReport::Status */
inline std::ostream&
operator << (std::ostream& s, const dueca::ReplayReport::Status& o)
{ return s << dueca::getString(o); }
/** Read an object of type ReplayReport::Status */
inline std::istream&
/** Read an enum of type ReplayReport::Status */
operator >> (std::istream& s, dueca::ReplayReport::Status& o)
{ std::string tmp; s >> tmp; dueca::readFromString(o, tmp); return s; }
PRINT_NS_END;

namespace std {
/** print to a stream. */
inline std::ostream &
operator << (std::ostream& s, const dueca::ReplayReport& o)
{ return o.print(s); }
};

// specialization of classname
template<>
const char* dueca::getclassname<dueca::ReplayReport>();
template <>
inline const char* dueca::getclassname<const dueca::ReplayReport>()
{ return dueca::getclassname<dueca::ReplayReport>(); }

#if !defined(__DCO_STANDALONE)
namespace dueca {
/** Template specialization, defines a trait that is needed if
    ReplayReport is ever used inside other dco objects. */
template <>
struct dco_nested<dueca::ReplayReport> : public dco_isnested { };
template <>
struct dco_nested<dueca::ReplayReport::Status> : public dco_isenum { };
};

#endif


