/* ------------------------------------------------------------------ */
/*      item            : Snapshot.hxx
        generated by    : Autobuild (abuild)
        date            : Mon 27 May 2024
        category        : header file
        description     : DUECA Communication Object (DCO)
                          automatically generated by dueca-codegen
        codegen version : 111
        language        : C++
        item            : Snapshot.dco
        made by         : Rene' van Paassen
        date            : 1301002
        description     : Generated from a hard-coded version
        copyright       : (c) 2016-2022 TUDelft-AE-C&S
                        : (c) 2022 René van Paassen
        license         : EUPL-1.2
        documentation   : DUECA_API

*/

#pragma once
#if !defined(__DCO_NOPACK)
namespace dueca {
class AmorphStore;
class AmorphReStore;
struct DataWriterArraySize;
};
#endif

#if !defined(__DCO_STANDALONE)
namespace dueca {
struct CommObjectDataTable;
};

#include <gencodegen.h>
#if GENCODEGEN != 111
#error "Generated Snapshot.hxx too old, please clean with 'make mrproper'"
#endif
#include <dueca/CommObjectTraits.hxx>
#endif
#include <iostream>

#include <smartstring.hxx>
#include <toml.hpp>
#include <NameSet.hxx>

namespace dueca {
 
/**  A Snapshot object can be used to send the data from a state
     snapshot, i.e. the data describing a (piece of) your model state
     in time, to a repository for snapshot, and conversely, for sending
     this data back to a SimulationModule, for restoration of an old
     state.
   
     The Snapshot object allocates a data buffer for you. It is possible
     to use an AmorphStore object to pack the data in this buffer, like:
   
     @code
       // make a snapshot to send later
       // note that "DoubleFile" is just one of the options for storage,
       // Binary and Base64 would also work here
       Snapshot *snap = new Snapshot(5 * sizeof(double), getNameSet(),
                                     Snapshot::DoubleFile);
       // and an AmorphStore to pack the data in there
       AmorphStore s(snap->accessData(), snap->getDataSize());
   
       // pack the state
       for (int ii = 5; ii--; ) {
         packData(s, my_x_copy[ii]);
       }
   
       // done, can now send off the snapshot.
     @endcode
   
     Alternatively, the snapshot can be packed with JSON or XML data.
     The "data" member in the snapshot is a smart string that can be
     easily fed with data, first a JSON example, using rapidjson:
   
     @code
       // somewhere appropriate, include json
       #include "rapidjson/writer.h"
       using namespace rapidjson;
   
       // make a snapshot, initially empty string
       Snapshot *snap = new Snapshot("", getNameSet(), Snapshot::JSON);
       // the data object in the snapshot can be used in combination with
       // a json writer, the following gives an appropriate standard writer:
       smartstring::json_string_writer writer(snap->data);
   
       // now code the data; using an array here. Of course you can use
       // other formats as well, e.g., Object. See the rapidjson documentation
       writer.StartArray()
       for (int ii = 5; ii--; ) {
         writer.Double(my_x_copy[ii]);
       }
       writer.EndArray();
   
       // done, can now send off the snapshot.
     @endcode
   
     The same can be done with XML, using pugixml. Of course, you might
     encounter code with the "old" XMLSnapshot approach, that produces
     equally valid XML
   
     @code
       // somewhere appropriate, include pugixml
       #include <pugixml.hpp>
       using namespace pugi;   // or prefix variables with pugi
   
       // make a snapshot, initially empty string
       Snapshot *snap = new Snapshot("", getNameSet(), Snapshot::XML);
   
       // the data object in the snapshot can be used in combination with
       // an xml writer, the following gives an appropriate standard writer:
       smartstring::xml_string_writer writer(snap->data);
   
       // now code the data; the convention/layout of the XML is your choice.
       // See also the pugixml documentation
       xml_document doc;
       xml_node root = doc.child("state")
       for (int ii = 5; ii--; ) {
         xml_node element = root.child("value");
         xml_attribute idx = element.attribute("idx");
         idx.set_value(ii);
         element.set_value(my_x_copy[ii]);
       }
   
       // this saves the created XML document to the smartstring in the
       // snapshot object.
       doc.save(writer);
   
       // done, can now send off the snapshot.
     @endcode
    */
struct Snapshot
{
  /** typedef for internal reference */
  typedef Snapshot __ThisDCOType__;

public:
  /** The name of this class. */
  static const char* const classname;


/** Enumerated type for an automatically generated object class */
  enum SnapCoding {
    /** old-style, no indication about coding. Treated as Base64 */
    UnSpecified,
    /** probably AmorphStore coded binary, stored in Base64 in toml */
    Base64,
    /** data directly encoded and stored as JSON string in inco toml */
    JSON,
    /** data directly encoded and stored as XML string in inco toml */
    XML,
    /** data encoded in AmorphStore, all floats, stored in toml */
    Floats,
    /** data encoded in AmorphStore, all doubles, stored in toml */
    Doubles,
    /** External binary file, binary storage, no information on format */
    BinaryFile,
    /** External ASCII formatted file, data AmorphStore packed as floats */
    FloatFile,
    /** External ASCII formatted file, data AmorphStore packed as double */
    DoubleFile,
    /** data directly encoded and stored as JSON string in external file */
    JSONFile,
    /** data directly encoded and stored as XML string in external file */
    XMLFile,
    /** data stored in Base64 in external file */
    Base64File
  };

  /** A place for the data, variable size. Use this with ASCII converted
      data (XML, JSON), or use an AmorphStore to pack as binary data. */
  dueca::smartstring data;

  /** Identifies the object that sent the snapshot */
  NameSet originator;

  /** Snapshot encoding, if specified */
  SnapCoding coding;

public:
  /** a "magic" number, hashed out of the class definition,
      that will be used to check consistency of the sent objects
      across the dueca nodes. */
  static const uint32_t magic_check_number;

  /** default constructor. */
  Snapshot();

  /** Constructor with arguments */
  Snapshot(
        const dueca::smartstring& data,
        const NameSet& originator,
        const SnapCoding& coding);

  /** copy constructor. */
  Snapshot(const Snapshot& o);

#if !defined(__DCO_NOPACK)
  /** constructor to restore an Snapshot from amorphous storage. */
  Snapshot(::dueca::AmorphReStore& r);
#endif

  /** destructor. */
  ~Snapshot();

#if !defined(__DCO_STANDALONE)
  /** new operator "new", which places objects not on a
      heap, but in one of the memory arenas. This to speed up
      memory management. */
  static void* operator new(size_t size);

  /** new operator "delete", to go with the new version
      of operator new. */
  static void operator delete(void* p);

  /** placement "new", needed for stl. */
  inline static void* operator new(size_t size, Snapshot*& o)
  { return reinterpret_cast<void*>(o); }
#endif

#if !defined(__DCO_NOPACK)
  /** packs the Snapshot into amorphous storage. */
  void packData(::dueca::AmorphStore& s) const;

  /** packs the Snapshot into amorphous storage.
      only differences with a previous object are packed. */
  void packDataDiff(::dueca::AmorphStore& s, const Snapshot& ref) const;

  /** unpacks the Snapshot from an amorphous storage. */
  void unPackData(::dueca::AmorphReStore& s);

  /** unpacks the differences for Snapshot
      from an amorphous storage. */
  void unPackDataDiff(::dueca::AmorphReStore& s);
#endif

  /** Test for equality. */
  bool operator == (const Snapshot& o) const;

  /** Test for inequality. */
  inline bool operator != (const Snapshot& o) const
  { return !(*this == o); }

  /** Assignment operator. */
  Snapshot& operator=(const Snapshot& o);

  /** prints the Snapshot to a stream. */
  std::ostream & print(std::ostream& s) const;

// Additional custom code
#include "SnapshotExtra.hxx"
};

} 

#if !defined(__DCO_NOPACK)
/** pack the object into amorphous storage. */
inline void packData(::dueca::AmorphStore& s,
                     const dueca::Snapshot& o)
{ o.packData(s); }

/** pack the differences between this object and another
    into amorphous storage. */
inline void packDataDiff(dueca::AmorphStore& s,
                         const dueca::Snapshot& o,
                         const dueca::Snapshot& ref)
{ o.packDataDiff(s, ref); }

/** unpack the object from amorphous storage. */
inline void unPackData(::dueca::AmorphReStore& s,
                       dueca::Snapshot& o)
{ o.unPackData(s); }

/** unpack the differences to this object from storage. */
inline void unPackDataDiff(dueca::AmorphReStore& s,
                           dueca::Snapshot& o)
{ o.unPackDataDiff(s); }
#endif

namespace dueca {
  const char* const getString(const dueca::Snapshot::SnapCoding &o);
  void readFromString(dueca::Snapshot::SnapCoding &o, const std::string& s);
  void getFirst(dueca::Snapshot::SnapCoding &o);
  bool getNext(dueca::Snapshot::SnapCoding &o);

  /** This function retrieves the classname of an Snapshot::SnapCoding */
  template <>
  const char* getclassname<dueca::Snapshot::SnapCoding>();
}; // end namespace dueca

#if !defined(__DCO_NOPACK)
void packData(::dueca::AmorphStore& s,
              const dueca::Snapshot::SnapCoding &o);
void unPackData(::dueca::AmorphReStore& s,
                dueca::Snapshot::SnapCoding &o);
#endif

PRINT_NS_START;
/** Print an object of type Snapshot::SnapCoding */
inline std::ostream&
operator << (std::ostream& s, const dueca::Snapshot::SnapCoding& o)
{ return s << dueca::getString(o); }
/** Read an object of type Snapshot::SnapCoding */
inline std::istream&
/** Read an enum of type Snapshot::SnapCoding */
operator >> (std::istream& s, dueca::Snapshot::SnapCoding& o)
{ std::string tmp; s >> tmp; dueca::readFromString(o, tmp); return s; }
PRINT_NS_END;

namespace std {
/** print to a stream. */
inline std::ostream &
operator << (std::ostream& s, const dueca::Snapshot& o)
{ return o.print(s); }
};

// specialization of classname
template<>
const char* dueca::getclassname<dueca::Snapshot>();
template <>
inline const char* dueca::getclassname<const dueca::Snapshot>()
{ return dueca::getclassname<dueca::Snapshot>(); }

#if !defined(__DCO_STANDALONE)
namespace dueca {
/** Template specialization, defines a trait that is needed if
    Snapshot is ever used inside other dco objects. */
template <>
struct dco_nested<dueca::Snapshot> : public dco_isnested { };
template <>
struct dco_nested<dueca::Snapshot::SnapCoding> : public dco_isenum { };
};

#endif


