/* ------------------------------------------------------------------ */
/*      item            : HDFReplayConfig.hxx
        generated by    : Autobuild (abuild)
        date            : Mon 27 May 2024
        category        : header file
        description     : DUECA Communication Object (DCO)
                          automatically generated by dueca-codegen
        codegen version : 111
        language        : C++
        original item   : HDFReplayConfig.dco
        made by         : Rene' van Paassen
        date            : 210224
        api             : DUECA_API
        description     : Configuration/Coordination hdf replay
        copyright       : (c) 2021-2022 TUDelft-AE-C&S
                        : (c) 2022 René van Paassen
        license         : EUPL-1.2
*/

#pragma once
#if !defined(__DCO_NOPACK)
namespace dueca {
class AmorphStore;
class AmorphReStore;
struct DataWriterArraySize;
};
#endif

#if !defined(__DCO_STANDALONE)
namespace dueca {
struct CommObjectDataTable;
};

#include <gencodegen.h>
#if GENCODEGEN != 111
#error "Generated HDFReplayConfig.hxx too old, please clean with 'make mrproper'"
#endif
#include <dueca/CommObjectTraits.hxx>
#endif
#include <iostream>

#include <stringoptions.h>
#include <SimTime.hxx>

namespace dueca {
 
/** Configuration message for HDF5 logging.
   
    When configured to listen to config messages, the HDF5Replayer can
    switch files or re-use files when in HoldCurrent mode.
   
    Possible modes of operation:
   
    - Spool back in the currently used file; send a HDFReplayConfig event
      with empty filename.
   
    - Close the current file and open an new one, send the appropriate
      file name.
   
    - Replay can also start at a later point in the file; send a non-zero
      replay_skip, to start the data at the offset tick values specified in the
      replay skip.
    */
struct HDFReplayConfig
{
  /** typedef for internal reference */
  typedef HDFReplayConfig __ThisDCOType__;

public:
  /** The name of this class. */
  static const char* const classname;


  /** File name pattern for reading an HDF log, if empty, use previous
      file. */
  vstring filename;

  /** Data point to start reading in the data file, defined in DUECA
      granules. A value of 0 implies starting from the first detected
      stream data point */
  dueca::TimeTickType replay_start;

public:
  /** a "magic" number, hashed out of the class definition,
      that will be used to check consistency of the sent objects
      across the dueca nodes. */
  static const uint32_t magic_check_number;

  /** default constructor. */
  HDFReplayConfig();

  /** Constructor with arguments */
  HDFReplayConfig(
        const vstring& filename,
        const dueca::TimeTickType& replay_start);

  /** copy constructor. */
  HDFReplayConfig(const HDFReplayConfig& o);

#if !defined(__DCO_NOPACK)
  /** constructor to restore an HDFReplayConfig from amorphous storage. */
  HDFReplayConfig(::dueca::AmorphReStore& r);
#endif

  /** destructor. */
  ~HDFReplayConfig();

#if !defined(__DCO_STANDALONE)
  /** new operator "new", which places objects not on a
      heap, but in one of the memory arenas. This to speed up
      memory management. */
  static void* operator new(size_t size);

  /** new operator "delete", to go with the new version
      of operator new. */
  static void operator delete(void* p);

  /** placement "new", needed for stl. */
  inline static void* operator new(size_t size, HDFReplayConfig*& o)
  { return reinterpret_cast<void*>(o); }
#endif

#if !defined(__DCO_NOPACK)
  /** packs the HDFReplayConfig into amorphous storage. */
  void packData(::dueca::AmorphStore& s) const;

  /** packs the HDFReplayConfig into amorphous storage.
      only differences with a previous object are packed. */
  void packDataDiff(::dueca::AmorphStore& s, const HDFReplayConfig& ref) const;

  /** unpacks the HDFReplayConfig from an amorphous storage. */
  void unPackData(::dueca::AmorphReStore& s);

  /** unpacks the differences for HDFReplayConfig
      from an amorphous storage. */
  void unPackDataDiff(::dueca::AmorphReStore& s);
#endif

  /** Test for equality. */
  bool operator == (const HDFReplayConfig& o) const;

  /** Test for inequality. */
  inline bool operator != (const HDFReplayConfig& o) const
  { return !(*this == o); }

  /** Assignment operator. */
  HDFReplayConfig& operator=(const HDFReplayConfig& o);

  /** prints the HDFReplayConfig to a stream. */
  std::ostream & print(std::ostream& s) const;
};

} 

#if !defined(__DCO_NOPACK)
/** pack the object into amorphous storage. */
inline void packData(::dueca::AmorphStore& s,
                     const dueca::HDFReplayConfig& o)
{ o.packData(s); }

/** pack the differences between this object and another
    into amorphous storage. */
inline void packDataDiff(dueca::AmorphStore& s,
                         const dueca::HDFReplayConfig& o,
                         const dueca::HDFReplayConfig& ref)
{ o.packDataDiff(s, ref); }

/** unpack the object from amorphous storage. */
inline void unPackData(::dueca::AmorphReStore& s,
                       dueca::HDFReplayConfig& o)
{ o.unPackData(s); }

/** unpack the differences to this object from storage. */
inline void unPackDataDiff(dueca::AmorphReStore& s,
                           dueca::HDFReplayConfig& o)
{ o.unPackDataDiff(s); }
#endif

namespace std {
/** print to a stream. */
inline std::ostream &
operator << (std::ostream& s, const dueca::HDFReplayConfig& o)
{ return o.print(s); }
};

// specialization of classname
template<>
const char* dueca::getclassname<dueca::HDFReplayConfig>();
template <>
inline const char* dueca::getclassname<const dueca::HDFReplayConfig>()
{ return dueca::getclassname<dueca::HDFReplayConfig>(); }

#if !defined(__DCO_STANDALONE)
namespace dueca {
/** Template specialization, defines a trait that is needed if
    HDFReplayConfig is ever used inside other dco objects. */
template <>
struct dco_nested<dueca::HDFReplayConfig> : public dco_isnested { };
};

#endif


