/* ------------------------------------------------------------------ */
/*      item            : ReplicatorInfo.hxx
        generated by    : Autobuild (abuild)
        date            : Mon 27 May 2024
        category        : header file
        description     : DUECA Communication Object (DCO)
                          automatically generated by dueca-codegen
        codegen version : 111
        language        : C++
        original item   : ReplicatorInfo.dco
        made by         : Rene' van Paassen
        date            : 170205
        documentation   : DUECA_API
        description     : Information on replicator status
        copyright       : (c) 2017-2022 TUDelft-AE-C&S
                        : (c) 2022 René van Paassen
        license         : EUPL-1.2
*/

#pragma once
#if !defined(__DCO_NOPACK)
namespace dueca {
class AmorphStore;
class AmorphReStore;
struct DataWriterArraySize;
};
#endif

#if !defined(__DCO_STANDALONE)
namespace dueca {
struct CommObjectDataTable;
};

#include <gencodegen.h>
#if GENCODEGEN != 111
#error "Generated ReplicatorInfo.hxx too old, please clean with 'make mrproper'"
#endif
#include <dueca/CommObjectTraits.hxx>
#endif
#include <iostream>

#include <inttypes.h>
typedef uint16_t peer_id_t;
#include <inttypes.h>
typedef uint16_t entry_id_t;
#include <stringoptions.h>

namespace dueca {
 
/** Start information for a peer joining a replicator network.
    */
struct ReplicatorInfo
{
  /** typedef for internal reference */
  typedef ReplicatorInfo __ThisDCOType__;

public:
  /** The name of this class. */
  static const char* const classname;


/** defines type of message
 */
  enum MessageType {
    /** Addition of a new peer, writes peer id, name is net address, and
        network port in entry_id */
    AddPeer,
    /** Add an entry, name is entry label, channelname filled */
    AddEntry,
    /** Remove an entry, name not filled */
    RemoveEntry,
    /** Remove an peer, only peer id filled */
    RemovePeer,
    /** default value */
    Undefined
  };

  /** type of information message */
  MessageType mtype;

  /** peer id */
  peer_id_t peer_id;

  /** entry id */
  entry_id_t entry_id;

  /** name, used for peer internet address or channel entry label */
  vstring name;

  /** channel name */
  vstring channelname;

  /** data class of the entry */
  vstring dataclass;

public:
  /** a "magic" number, hashed out of the class definition,
      that will be used to check consistency of the sent objects
      across the dueca nodes. */
  static const uint32_t magic_check_number;

  /** default constructor. */
  ReplicatorInfo();

  /** Constructor with arguments */
  ReplicatorInfo(
        const MessageType& mtype,
        const peer_id_t& peer_id,
        const entry_id_t& entry_id,
        const vstring& name,
        const vstring& channelname,
        const vstring& dataclass);

  /** copy constructor. */
  ReplicatorInfo(const ReplicatorInfo& o);

#if !defined(__DCO_NOPACK)
  /** constructor to restore an ReplicatorInfo from amorphous storage. */
  ReplicatorInfo(::dueca::AmorphReStore& r);
#endif

  /** destructor. */
  ~ReplicatorInfo();

#if !defined(__DCO_STANDALONE)
  /** new operator "new", which places objects not on a
      heap, but in one of the memory arenas. This to speed up
      memory management. */
  static void* operator new(size_t size);

  /** new operator "delete", to go with the new version
      of operator new. */
  static void operator delete(void* p);

  /** placement "new", needed for stl. */
  inline static void* operator new(size_t size, ReplicatorInfo*& o)
  { return reinterpret_cast<void*>(o); }
#endif

#if !defined(__DCO_NOPACK)
  /** packs the ReplicatorInfo into amorphous storage. */
  void packData(::dueca::AmorphStore& s) const;

  /** packs the ReplicatorInfo into amorphous storage.
      only differences with a previous object are packed. */
  void packDataDiff(::dueca::AmorphStore& s, const ReplicatorInfo& ref) const;

  /** unpacks the ReplicatorInfo from an amorphous storage. */
  void unPackData(::dueca::AmorphReStore& s);

  /** unpacks the differences for ReplicatorInfo
      from an amorphous storage. */
  void unPackDataDiff(::dueca::AmorphReStore& s);
#endif

  /** Test for equality. */
  bool operator == (const ReplicatorInfo& o) const;

  /** Test for inequality. */
  inline bool operator != (const ReplicatorInfo& o) const
  { return !(*this == o); }

  /** Assignment operator. */
  ReplicatorInfo& operator=(const ReplicatorInfo& o);

  /** prints the ReplicatorInfo to a stream. */
  std::ostream & print(std::ostream& s) const;
};

} 

#if !defined(__DCO_NOPACK)
/** pack the object into amorphous storage. */
inline void packData(::dueca::AmorphStore& s,
                     const dueca::ReplicatorInfo& o)
{ o.packData(s); }

/** pack the differences between this object and another
    into amorphous storage. */
inline void packDataDiff(dueca::AmorphStore& s,
                         const dueca::ReplicatorInfo& o,
                         const dueca::ReplicatorInfo& ref)
{ o.packDataDiff(s, ref); }

/** unpack the object from amorphous storage. */
inline void unPackData(::dueca::AmorphReStore& s,
                       dueca::ReplicatorInfo& o)
{ o.unPackData(s); }

/** unpack the differences to this object from storage. */
inline void unPackDataDiff(dueca::AmorphReStore& s,
                           dueca::ReplicatorInfo& o)
{ o.unPackDataDiff(s); }
#endif

namespace dueca {
  const char* const getString(const dueca::ReplicatorInfo::MessageType &o);
  void readFromString(dueca::ReplicatorInfo::MessageType &o, const std::string& s);
  void getFirst(dueca::ReplicatorInfo::MessageType &o);
  bool getNext(dueca::ReplicatorInfo::MessageType &o);

  /** This function retrieves the classname of an ReplicatorInfo::MessageType */
  template <>
  const char* getclassname<dueca::ReplicatorInfo::MessageType>();
}; // end namespace dueca

#if !defined(__DCO_NOPACK)
void packData(::dueca::AmorphStore& s,
              const dueca::ReplicatorInfo::MessageType &o);
void unPackData(::dueca::AmorphReStore& s,
                dueca::ReplicatorInfo::MessageType &o);
#endif

PRINT_NS_START;
/** Print an object of type ReplicatorInfo::MessageType */
inline std::ostream&
operator << (std::ostream& s, const dueca::ReplicatorInfo::MessageType& o)
{ return s << dueca::getString(o); }
/** Read an object of type ReplicatorInfo::MessageType */
inline std::istream&
/** Read an enum of type ReplicatorInfo::MessageType */
operator >> (std::istream& s, dueca::ReplicatorInfo::MessageType& o)
{ std::string tmp; s >> tmp; dueca::readFromString(o, tmp); return s; }
PRINT_NS_END;

namespace std {
/** print to a stream. */
inline std::ostream &
operator << (std::ostream& s, const dueca::ReplicatorInfo& o)
{ return o.print(s); }
};

// specialization of classname
template<>
const char* dueca::getclassname<dueca::ReplicatorInfo>();
template <>
inline const char* dueca::getclassname<const dueca::ReplicatorInfo>()
{ return dueca::getclassname<dueca::ReplicatorInfo>(); }

#if !defined(__DCO_STANDALONE)
namespace dueca {
/** Template specialization, defines a trait that is needed if
    ReplicatorInfo is ever used inside other dco objects. */
template <>
struct dco_nested<dueca::ReplicatorInfo> : public dco_isnested { };
template <>
struct dco_nested<dueca::ReplicatorInfo::MessageType> : public dco_isenum { };
};

#endif


